exec("./barricades.cs");
exec("./items.cs");
exec("./crafting.cs");
exec("./zombieAttackOverride.cs");

function servercmdb(%client, %num)
{
	switch(%num)
	{
		case 1:
			%client.player.setBarricade(1);
		case 2:
			%client.player.setBarricade(2);
		case 3: 
			%client.player.setBarricade(3);
	}
}

function staticShape::getDamage(%this)
{
	return %this.damage;
}

function staticShape::setDamage(%this, %damage)
{
	%this.damage = %damage;
}

function staticShape::repair(%this)
{
	%rr = %this.getDataBlock().repairRate;
	%dmg = %this.getDamageCent();
	%lmt = 100 - %this.getLimitCent();
	if(%dmg <= 0)
	{
		%dmg = 1;
	}
	%green = %dmg / 100;
	%red = 1.0 - %green;
		
	if(%dmg != 100)
	{
		if(%lmt != 100)
		{
			%this.setShapeName("Barricade (" @ %dmg @ "% health) - " @ %lmt @ "% repair limit");
			if(%this.damage - %rr < 0)
			{
				%this.setDamage(0);
			}
			else
			{
				%this.setDamage(%this.damage - %rr);
			}
			%this.limit = %this.limit + %rr;
		}
	}
	else
	{
		%this.setShapeName("Barricade (" @ %dmg @ "% health)");
	}

	%this.setShapeNameColor(%red SPC %green @ " 0 1");
	cancel(%this.renameSch);
	%this.renameSch = %this.schedule(1500, "setShapeName", "Barricade (" @ %dmg @ "% health)");
}

function staticShape::getDamageCent(%this)
{	
	return 100 - mFloatLength((%this.damage / %this.getDataBlock().maxDamage), 2) * 100;
}

function staticShape::getLimitCent(%this)
{	
	return 100 - mFloatLength((%this.limit / 100), 2) * 100;
}

function getDistFromGround(%start)
{
	%x1 = getword(%start, 0);
	%y1 = getword(%start, 1);
	%z1 = getword(%start, 2);
	%end = vectorAdd(%start, %x1 SPC %y1 SPC (%z1 - 1000)); 
	%typemasks = $Typemasks::FxBrickObjectType | $Typemasks::TerrainObjectType | $TypeMasks::StaticObjectType;
	%ray = containerRaycast(%start, %end, %typemasks, 0);
	%pos = posFromRaycast(%ray);
	return vectorDist(%start, %pos);
}

function player::setBarricade(%player, %type)
{
	%client = %player.client;
	%typemasks = $Typemasks::FxBrickObjectType | $Typemasks::TerrainObjectType | $TypeMasks::StaticObjectType;
	%a = vectorAdd(vectorScale(vectorNormalize(%player.getEyeVector()),4),%player.getEyePoint());

	%target = containerRayCast(%player.getEyePoint(),%a,%typemasks,%player);
	
	%obj = firstword(%target);
	
	if(!isObject(%obj))
	{
		commandtoclient(%client, 'centerprint', "\c6Error placing barricade: \n\c6You need to be looking at the ground.", 2);
		return false;
	}
	
	%targetpos = posfromraycast(%target);
		
	%x = getword(%targetpos, 0);
	%y = getword(%targetpos, 1);
	%z = getword(%targetpos, 2);
	
	%x1 = getword(%player.getEyePoint(), 0);
	%y1 = getword(%player.getEyePoint(), 1);
	%z1 = getword(%player.getEyePoint(), 2);
		
	%x2 = (%x + %x1) / 2;
	%y2 = (%y + %y1) / 2;
	%z2 = (%z + %z1) / 2;
	
	%b = %x2 SPC %y2 SPC %z2; 

	if(getDistFromGround(%b) > 1.2)
	{
		commandtoclient(%client, 'centerprint', "\c6You can only place barricades on the ground.", 2);
		return false;
	}
	
	switch(%type)
	{
		case 1:
			%shape = new StaticShape()
			{
				datablock = "barricade_weak"; //datablock (shape etc)
				position = %targetPos; //position
				rotation = %player.rotation; //rotation
				damage = 0; //damage
				limit = 0; //repair limit
			};
		case 2:
			%shape = new StaticShape()
			{
				datablock = "barricade_medium";
				position = %targetPos;
				rotation = %player.rotation;
				damage = 0;
				limit = 0;
			};
		case 3:
			%shape = new StaticShape()
			{
				datablock = "barricade_strong";
				position = %targetPos;
				rotation = %player.rotation;
				damage = 0;
				limit = 0;
			};
	}
	
	new Projectile()
	{
		initialPosition = %targetpos;
		datablock = "spawnCadeProjectile";
	};

	%shape.setShapeName("Barricade (100% health)");
	%shape.setShapeNameColor("0 1 0 1");
	%shape.setShapeNameDistance(10);
	return true;
}

package barricade_server
{
	function ProjectileData::onCollision(%this,%obj,%col,%fade,%pos,%norm)
	{
		Parent::onCollision(%this,%obj,%col,%fade,%pos,%norm);
		if(%col.getClassName() $= "staticShape")
		{
			%d = %col.getDataBlock();
			if(%d.isBarricade)
			{
				%damage = %col.getDamage();
				if(%damage >= %d.maxDamage)
				{
					switch$(%col.getDataBlock().getName())
					{
						case "barricade_weak":
							%data = "barricade_weak_debrisProjectile";
						case "barricade_medium":
							%data = "barricade_medium_debrisProjectile";
						case "barricade_strong":
							%data = "barricade_strong_debrisProjectile";
					}
					new Projectile()
					{
						datablock = %data;
						initialPosition = %col.getPosition();
					};
					%col.delete();
					return;
				}
				else
				{
					%wear = %d.wearResist;
					%col.setDamage(%damage + (%this.directDamage / %wear));
					%dmg = %col.getDamageCent();
					if(%dmg <= 0)
					{
						%dmg = 1;
					}
					%green = %dmg / 100;
					%red = 1.0 - %green;
					
					%col.setShapeName("Barricade (" @ %dmg @ "% health)");
					%col.setShapeNameColor(%red SPC %green @ " 0 1");
				}
			}
		}
	}
};
activatepackage(barricade_server);